﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Data;

namespace CTHFramework
{
    /// <summary>
    /// 程序实现分页
    /// </summary>
    public class Paging
    {
        /// <summary>
        /// 分页 
        /// </summary>
        /// <param name="dataSource">数据源</param>
        /// <param name="pageIndex">第几页</param>
        /// <param name="pageSize">每页显示条数</param>
        /// <returns>指定页得数据</returns>
        public static DataTable GetPagingData(DataTable dataSource, int pageIndex, int pageSize)
        {
            if (pageIndex <= 0 || pageSize <= 0)
            {
                throw new IndexOutOfRangeException("参数超出数组范围!");
            }
              
            if (dataSource == null || dataSource.Rows.Count == 0)
            {
                return null;
            }

            if ((pageIndex - 1) * pageSize >= dataSource.Rows.Count)
            {
                return null;
            }

            if (dataSource.Rows.Count <= pageSize)
            {
                return dataSource;
            }

            int start = pageIndex > 1 ? start = (pageIndex - 1) * pageSize : 0;

            int end = start + pageSize;
            if (end > dataSource.Rows.Count)
            {
                end = dataSource.Rows.Count;
            }

            DataTable dt = dataSource.Clone();
            int columnsCount = dt.Columns.Count;
            DataRow dr, drSource;
            for (int i = start; i < end; i++)
            {
                dr = dt.NewRow();
                drSource = dataSource.Rows[i];
                for (int j = 0; j < columnsCount; j++)
                {
                    dr[j] = drSource[j];
                }
                dt.Rows.Add(dr);
            }
            return dt;
        }


    }
}
